"""
FunchoData SDK基本使用示例
"""

import fc

def main():
    # 设置API密钥
    print("设置API密钥...")
    fc.set_token('your_api_key_here')  # 请替换为您的API密钥
    
    # 示例1：基本数据获取
    print("\n=== 示例1：基本数据获取 ===")
    try:
        df = fc.get_df('para_main', params={'page': '1', 'size': '10'})
        print(f"获取到 {len(df)} 条数据")
        print("列名:", df.columns.tolist())
        print("前5行数据:")
        print(df.head())
    except Exception as e:
        print(f"错误: {e}")
    
    # 示例2：指定字段查询
    print("\n=== 示例2：指定字段查询 ===")
    try:
        params = {
            'page': '1',
            'size': '5',
            'para_clas_cd': ''
        }
        fields = ['id', 'para_clas_cd']
        
        df = fc.get_df('para_main', params=params, fields=fields)
        print(f"获取到 {len(df)} 条数据")
        print(df)
    except Exception as e:
        print(f"错误: {e}")
    
    # 示例3：获取JSON格式数据
    print("\n=== 示例3：获取JSON格式数据 ===")
    try:
        data = fc.get_json('para_main', params={'page': '1', 'size': '3'})
        print("JSON数据结构:")
        print(f"- success: {data.get('success', 'N/A')}")
        print(f"- message: {data.get('message', 'N/A')}")
        print(f"- data count: {len(data.get('data', []))}")
        if data.get('data'):
            print("第一条数据:", data['data'][0])
    except Exception as e:
        print(f"错误: {e}")

if __name__ == "__main__":
    main() 


"""
FunchoData SDK异常类
"""


class FunchoException(Exception):
    """FunchoData SDK基础异常类"""
    pass


class AuthenticationError(FunchoException):
    """身份验证错误"""
    pass


class APIError(FunchoException):
    """API请求错误"""
    pass


class DataError(FunchoException):
    """数据处理错误"""
    pass


class NetworkError(FunchoException):
    """网络错误"""
    pass 


"""
FunchoData API客户端
"""

import json
import time
import requests
import pandas as pd
from typing import Dict, List, Optional, Any
from .exceptions import FunchoException, AuthenticationError, APIError


class FunchoClient:
    """FunchoData API客户端"""
    
    def __init__(self, token: str = None, base_url: str = None):
        """初始化客户端
        
        Args:
            token (str, optional): API密钥
            base_url (str, optional): 基础URL
        """
        self.token = token
        self.base_url = base_url or "https://www.funchodata.com"
        self.session = requests.Session()
        
    def set_token(self, token: str):
        """设置API密钥"""
        self.token = token
        
    def set_base_url(self, base_url: str):
        """设置基础URL"""
        self.base_url = base_url
        
    def _get_headers(self) -> Dict[str, str]:
        """获取请求头"""
        if not self.token:
            raise AuthenticationError("未设置API密钥，请先调用 fc.set_token('your_api_key')")
            
        return {
            "Content-Type": "application/json",
            "api-key": self.token
        }
    
    def _make_request(self, form_code: str, params: Dict = None, fields: List[str] = None) -> Dict:
        """发送API请求（支持自动重试）
        
        Args:
            form_code (str): 表单编码
            params (dict): 请求参数
            fields (list): 字段列表
            
        Returns:
            dict: API响应数据
        """
        url = f"{self.base_url}/c-api/form/v1/dataframe/{form_code}"
        headers = self._get_headers()
        
        # 构建请求体
        request_data = {}
        
        if params:
            request_data["params"] = params
        else:
            request_data["params"] = {"page": "1", "size": "200"}
            
        if fields:
            request_data["fields"] = fields
        
        # 重试配置
        max_retries = 3  # 最大重试次数
        retry_count = 0
        base_delay = 2  # 基础延迟时间（秒），总等待时间: 4+8+16=28秒
        timeout = 30  # 单次请求超时时间（秒）
        
        while retry_count <= max_retries:
            try:
                response = self.session.post(url, json=request_data, headers=headers, timeout=timeout)
                
                if response.status_code == 200:
                    data = response.json()
                    # 检查新的响应格式
                    if data.get('code') != 200:
                        if data.get('code') == 401:
                            raise AuthenticationError(f"认证失败: {data.get('msg', 'API密钥无效或已过期')}")
                        else:
                            raise APIError(f"API返回错误: {data.get('msg', '未知错误')} (code: {data.get('code')})")
                    return data
                elif response.status_code == 401:
                    raise AuthenticationError("HTTP 401: API密钥无效或已过期")
                elif response.status_code == 404:
                    raise APIError(f"HTTP 404: 表单编码 '{form_code}' 不存在")
                else:
                    raise APIError(f"请求失败，状态码: {response.status_code}, 错误信息: {response.text}")
                    
            except (requests.exceptions.Timeout, 
                    requests.exceptions.ConnectionError,
                    requests.exceptions.RequestException) as e:
                retry_count += 1
                
                # 如果已经达到最大重试次数，抛出异常
                if retry_count > max_retries:
                    if isinstance(e, requests.exceptions.Timeout):
                        raise APIError(f"请求超时，已重试{max_retries}次仍失败")
                    elif isinstance(e, requests.exceptions.ConnectionError):
                        raise APIError(f"网络连接错误，已重试{max_retries}次仍失败")
                    else:
                        raise APIError(f"请求异常: {str(e)}，已重试{max_retries}次仍失败")
                
                # 计算指数退避延迟时间: 2^retry_count * base_delay
                delay = (2 ** retry_count) * base_delay
                
                # 记录重试信息（可选：后续可以添加日志记录）
                error_msg = str(e)
                if isinstance(e, requests.exceptions.Timeout):
                    error_type = "超时"
                elif isinstance(e, requests.exceptions.ConnectionError):
                    error_type = "连接错误"
                else:
                    error_type = "请求异常"
                
                print(f"[重试 {retry_count}/{max_retries}] {error_type}: {error_msg}，{delay}秒后重试...")
                
                # 等待后重试
                time.sleep(delay)
            
            except (AuthenticationError, APIError) as e:
                # 对于认证错误和API业务错误，不进行重试，直接抛出
                raise e
    
    def get_json(self, form_code: str, params: Dict = None, fields: List[str] = None, **kwargs) -> Dict:
        """获取JSON格式数据
        
        Args:
            form_code (str): 表单编码
            params (dict): 查询参数
            fields (list): 字段列表
            
        Returns:
            dict: JSON数据
        """
        # 合并kwargs到params
        if kwargs:
            if params is None:
                params = {}
            params.update(kwargs)
            
        return self._make_request(form_code, params, fields)
    
    def get_df(self, form_code: str, params: Dict = None, fields: List[str] = None, **kwargs) -> pd.DataFrame:
        """获取DataFrame格式数据
        
        Args:
            form_code (str): 表单编码
            params (dict): 查询参数
            fields (list): 字段列表
            
        Returns:
            pandas.DataFrame: 数据框
        """
        # 合并kwargs到params
        if kwargs:
            if params is None:
                params = {}
            params.update(kwargs)
            
        data = self._make_request(form_code, params, fields)
        
        # 转换为DataFrame
        if 'data' in data:
            if data['data'] is None:
                # 空数据时返回空DataFrame
                if fields:
                    return pd.DataFrame(columns=fields)
                else:
                    return pd.DataFrame()
            elif isinstance(data['data'], list):
                # 直接是列表格式：[{"col1": "val1"}, ...]
                if len(data['data']) > 0:
                    return pd.DataFrame(data['data'])
                else:
                    # 空数据时返回空DataFrame
                    if fields:
                        return pd.DataFrame(columns=fields)
                    else:
                        return pd.DataFrame()
            elif isinstance(data['data'], dict):
                # 处理字典格式的data，支持多种结构
                if 'list' in data['data']:
                    rows = data['data']['list']
                    
                    if len(rows) > 0:
                        # 检查list中的元素类型
                        if isinstance(rows[0], dict):
                            # 新格式：list中直接是字典 [{"id": "1", "name": "test"}, ...]
                            df = pd.DataFrame(rows)
                        elif isinstance(rows[0], (list, tuple)):
                            # 旧格式：list中是数组，需要fields字段 [["1", "test"], ...]
                            if 'fields' in data['data']:
                                field_names = data['data']['fields']
                                records = []
                                for row in rows:
                                    record = dict(zip(field_names, row))
                                    records.append(record)
                                df = pd.DataFrame(records)
                            else:
                                raise APIError("API返回的数据格式不正确：list中是数组但缺少fields字段")
                        else:
                            raise APIError(f"API返回的list元素类型不支持: {type(rows[0])}")
                        
                        # 如果指定了字段筛选，则只返回指定字段
                        if fields:
                            available_fields = [f for f in fields if f in df.columns]
                            if available_fields:
                                df = df[available_fields]
                            else:
                                # 如果指定的字段都不存在，返回空DataFrame
                                return pd.DataFrame(columns=fields)
                        
                        return df
                    else:
                        # 空数据时返回空DataFrame
                        if fields:
                            return pd.DataFrame(columns=fields)
                        else:
                            return pd.DataFrame()
                else:
                    raise APIError("API返回的data字段格式不正确，缺少'list'字段")
            else:
                raise APIError(f"API返回的data字段类型不正确: {type(data['data'])}")
        else:
            raise APIError("API返回的数据中没有'data'字段")
    
    def get_all_pages(self, form_code: str, params: Dict = None, fields: List[str] = None, 
                      max_pages: int = None) -> pd.DataFrame:
        """获取所有分页数据
        
        Args:
            form_code (str): 表单编码
            params (dict): 查询参数
            fields (list): 字段列表
            max_pages (int): 最大页数限制
            
        Returns:
            pandas.DataFrame: 合并后的数据框
        """
        if params is None:
            params = {}
            
        # 设置默认分页参数
        params.setdefault('page', '1')
        params.setdefault('size', '200')
        
        all_data = []
        page = int(params['page'])
        
        while True:
            params['page'] = str(page)
            data = self._make_request(form_code, params, fields)
            
            if 'data' in data and data['data'] is not None:
                current_data = []
                
                if isinstance(data['data'], list):
                    # 直接是列表格式
                    current_data = data['data']
                elif isinstance(data['data'], dict) and 'list' in data['data']:
                    rows = data['data'].get('list', [])
                    
                    if len(rows) > 0:
                        if isinstance(rows[0], dict):
                            # 新格式：list中直接是字典
                            current_data = rows
                        elif isinstance(rows[0], (list, tuple)):
                            # 旧格式：list中是数组，需要fields字段
                            field_names = data['data'].get('fields', [])
                            for row in rows:
                                record = dict(zip(field_names, row))
                                current_data.append(record)
                        else:
                            # 未知格式，跳过
                            pass
                
                if len(current_data) == 0:
                    break  # 没有更多数据
                    
                all_data.extend(current_data)
                
                # 检查是否还有更多页
                if len(current_data) < int(params['size']):
                    break  # 当前页数据少于页面大小，说明是最后一页
                    
                page += 1
                
                # 检查最大页数限制
                if max_pages and page > max_pages:
                    break
            else:
                break
                
        if all_data:
            return pd.DataFrame(all_data)
        else:
            if fields:
                return pd.DataFrame(columns=fields)
            else:
                return pd.DataFrame() 


"""
FunchoData SDK - 简化的数据获取工具

使用方法:
    import fc
    
    # 设置API密钥
    fc.set_token('your_api_key')
    
    # 获取数据
    df = fc.get_df('para_main', params={'para_clas_cd': ''}, fields=['id', 'para_clas_cd'])
"""

from .client import FunchoClient
from .exceptions import FunchoException, AuthenticationError, APIError

__version__ = "1.0.1"
__author__ = "FunchoData"

# 全局客户端实例
_client = FunchoClient()

def set_token(token):
    """设置API密钥
    
    Args:
        token (str): API密钥
    """
    _client.set_token(token)

def set_base_url(base_url):
    """设置基础URL（仅支持生产环境）
    
    Args:
        base_url (str): 基础URL，仅支持生产环境 https://www.funchodata.com
    """
    _client.set_base_url(base_url)

def get_df(form_code, params=None, fields=None, **kwargs):
    """获取DataFrame数据
    
    Args:
        form_code (str): 表单编码
        params (dict, optional): 查询参数，包含分页参数和条件参数
        fields (list, optional): 需要返回的字段列表
        **kwargs: 其他参数
        
    Returns:
        pandas.DataFrame: 返回的数据框
        
    Example:
        >>> import fc
        >>> fc.set_token('your_api_key')
        >>> df = fc.get_df('para_main', params={'page': '1', 'size': '200'}, fields=['id', 'para_clas_cd'])
    """
    return _client.get_df(form_code, params, fields, **kwargs)

def get_json(form_code, params=None, fields=None, **kwargs):
    """获取JSON数据
    
    Args:
        form_code (str): 表单编码
        params (dict, optional): 查询参数
        fields (list, optional): 需要返回的字段列表
        **kwargs: 其他参数
        
    Returns:
        dict: 返回的JSON数据
    """
    return _client.get_json(form_code, params, fields, **kwargs)

def get_all_pages(form_code, params=None, fields=None, max_pages=None):
    """获取所有分页数据
    
    Args:
        form_code (str): 表单编码
        params (dict, optional): 查询参数
        fields (list, optional): 需要返回的字段列表
        max_pages (int, optional): 最大页数限制
        
    Returns:
        pandas.DataFrame: 合并后的数据框
    """
    return _client.get_all_pages(form_code, params, fields, max_pages)

# 向后兼容的别名
pro_api = get_df 


"""
FunchoData SDK高级使用示例
"""

import fc
from fc.exceptions import AuthenticationError, APIError
import pandas as pd

def example_error_handling():
    """错误处理示例"""
    print("=== 错误处理示例 ===")
    
    # 测试无效API密钥
    try:
        fc.set_token('invalid_token')
        df = fc.get_df('para_main')
    except AuthenticationError as e:
        print(f"认证错误（预期）: {e}")
    except Exception as e:
        print(f"其他错误: {e}")
    
    # 恢复正确的token
    fc.set_token('your_api_key_here')
    
    # 测试不存在的表单
    try:
        df = fc.get_df('non_existent_form')
    except APIError as e:
        print(f"API错误（预期）: {e}")
    except Exception as e:
        print(f"其他错误: {e}")

def example_pagination():
    """分页查询示例"""
    print("\n=== 分页查询示例 ===")
    
    try:
        # 获取第一页数据
        page1 = fc.get_df('para_main', params={'page': '1', 'size': '5'})
        print(f"第1页: {len(page1)} 条数据")
        
        # 获取第二页数据
        page2 = fc.get_df('para_main', params={'page': '2', 'size': '5'})
        print(f"第2页: {len(page2)} 条数据")
        
        # 合并多页数据
        combined = pd.concat([page1, page2], ignore_index=True)
        print(f"合并后: {len(combined)} 条数据")
        
    except Exception as e:
        print(f"分页查询错误: {e}")

def example_field_selection():
    """字段选择示例"""
    print("\n=== 字段选择示例 ===")
    
    try:
        # 获取所有字段
        df_all = fc.get_df('para_main', params={'page': '1', 'size': '3'})
        print(f"所有字段: {df_all.columns.tolist()}")
        
        # 只获取指定字段
        fields = ['id', 'para_clas_cd']
        df_selected = fc.get_df('para_main', 
                               params={'page': '1', 'size': '3'}, 
                               fields=fields)
        print(f"选择字段: {df_selected.columns.tolist()}")
        print(df_selected)
        
    except Exception as e:
        print(f"字段选择错误: {e}")

def example_data_processing():
    """数据处理示例"""
    print("\n=== 数据处理示例 ===")
    
    try:
        df = fc.get_df('para_main', params={'page': '1', 'size': '20'})
        
        print(f"数据形状: {df.shape}")
        print(f"数据类型:\n{df.dtypes}")
        
        # 数据统计
        if len(df) > 0:
            print(f"数值列统计:")
            numeric_cols = df.select_dtypes(include=['number']).columns
            if len(numeric_cols) > 0:
                print(df[numeric_cols].describe())
            
            # 分类统计
            categorical_cols = df.select_dtypes(include=['object']).columns
            for col in categorical_cols[:2]:  # 只显示前两个分类列
                print(f"\n{col} 值分布:")
                print(df[col].value_counts().head())
                
    except Exception as e:
        print(f"数据处理错误: {e}")


def main():
    """主函数"""
    print("FunchoData SDK高级使用示例")
    print("=" * 50)
    
    # 设置API密钥
    fc.set_token('your_api_key_here')  # 请替换为您的API密钥
    
    # 运行各种示例
    example_error_handling()
    example_pagination() 
    example_field_selection()
    example_data_processing()

if __name__ == "__main__":
    main() 


"""
FunchoData SDK安装配置文件
"""

from setuptools import setup, find_packages
import os

# 读取README文件
def read_readme():
    readme_path = os.path.join(os.path.dirname(__file__), 'README.md')
    if os.path.exists(readme_path):
        with open(readme_path, 'r', encoding='utf-8') as f:
            return f.read()
    return ""

# 读取版本信息
def get_version():
    version_file = os.path.join(os.path.dirname(__file__), 'fc', '__init__.py')
    with open(version_file, 'r', encoding='utf-8') as f:
        for line in f:
            if line.startswith('__version__'):
                return line.split('=')[1].strip().strip('"\'')
    return "1.0.0"

setup(
    name="funchodata",
    version=get_version(),
    author="FunchoData",
    description="FunchoData官方Python SDK - 简化的金融数据获取工具",
    long_description=read_readme(),
    long_description_content_type="text/markdown",
    url="https://www.funchodata.com",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Financial and Insurance Industry",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Office/Business :: Financial",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.6",
    install_requires=[
        "requests>=2.25.0",
        "pandas>=1.0.0",
    ],
    extras_require={
        "dev": [
            "pytest>=6.0.0",
            "pytest-cov>=2.10.0",
            "black>=21.0.0",
            "flake8>=3.8.0",
        ],
    },
    keywords="finance, financial-data, api, sdk, pandas, dataframe, funchodata, 金融数据, 数据接口",
    project_urls={
        "官网": "https://www.funchodata.com",
    },
) 